/* 
 * This file is part of hanga studio.
 *
 * Copyright (C) 2007 hanga studio Project.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
 */
package kr.sohalee.hangastudio.model.mail.message;

import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import javax.mail.BodyPart;
import javax.mail.Flags;
import javax.mail.Folder;
import javax.mail.Message;
import javax.mail.Multipart;
import javax.mail.Part;
import javax.mail.Session;
import javax.mail.Flags.Flag;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;
import javax.mail.internet.MimeUtility;
import javax.mail.search.AndTerm;
import javax.mail.search.ComparisonTerm;
import javax.mail.search.FromStringTerm;
import javax.mail.search.OrTerm;
import javax.mail.search.SearchTerm;
import javax.mail.search.SentDateTerm;
import javax.mail.search.SubjectTerm;

import kr.sohalee.hangastudio.constants.MailConstants;
import kr.sohalee.hangastudio.core.exception.KnownException;
import kr.sohalee.hangastudio.core.system.SSLInitializer;
import kr.sohalee.hangastudio.core.util.BooleanUtil;
import kr.sohalee.hangastudio.core.util.CollectionUtil;
import kr.sohalee.hangastudio.core.util.JavaMailUtil;
import kr.sohalee.hangastudio.core.util.StringUtil;
import kr.sohalee.hangastudio.model.mail.MailStoreBean;
import kr.sohalee.hangastudio.model.mail.acc.AccRecvBean;
import kr.sohalee.hangastudio.model.mail.acc.AccSendBean;

import org.apache.log4j.Logger;

import com.sun.mail.smtp.SMTPTransport;

/**
 * 메일 메세지 DAO 구상 클래스 (JavaMail)
 * 
 * @author <a href="mailto:sohalee@dreamwiz.com">Lee, Kyung-rok</a>
 * 
 */
public class MessageDaoImplJavaMail implements MessageDao {

	private static Logger logger = Logger
			.getLogger(MessageDaoImplJavaMail.class);

	private Message getJavaMailMessage(Session session,
			AccRecvBean accRecvBean, MessageBean messageBean,
			Folder forwardFolder) throws Exception {

		String encoding = messageBean.getEncoding();
		if (StringUtil.isEmpty(encoding)) {
			encoding = System.getProperty("file.encoding");
		}

		Message message = new MimeMessage(session);

		message.setFrom(new InternetAddress(messageBean.getFromAddress(),
				messageBean.getFromName(), encoding));
		message.setSubject(MimeUtility.encodeText(messageBean.getSubject(),
				encoding, "B"));
		// 받는사람들의 주소를 넣는다.
		setRecipients(message, messageBean);

		// 본문내용을 BodyPart로 생성하여 반환.
		BodyPart contentPart = JavaMailUtil.getContentBodyPart(messageBean
				.getContent(), messageBean.getContentMimeType(), encoding);

		/*
		 * Forward Message (전달시 원본)
		 */

		Message forwardMessage = null;
		if (BooleanUtil.booleanValue(messageBean.getForward())
				&& messageBean.getFolderName() != null) {

			if (forwardFolder == null || !forwardFolder.isOpen()) {
				throw new KnownException("folder not opened for forward");
			}

			forwardMessage = forwardFolder.getMessage(messageBean.getNo()
					.intValue());
		}

		// 첨부파일이나 forward(전달)메세지가 있는경우
		if (!CollectionUtil.isEmpty(messageBean.getAttachList())
				|| forwardMessage != null) {

			MimeMultipart multipart = new MimeMultipart();

			// 본문 BodyPart를 넣는다.
			multipart.addBodyPart(contentPart);
			// forward 메세지를 multipart에 넣는다.
			if (forwardMessage != null) {
				multipart.addBodyPart(JavaMailUtil
						.getMessageRfc822BodyPart(forwardMessage));
			}
			// 첨부파일을 multipart에 넣는다.
			setAttaches(multipart, messageBean, encoding);

			message.setContent(multipart);

			multipart = null;
		} else {

			// multipart/alternative인경우
			if (contentPart.getContent() instanceof MimeMultipart) {
				message.setContent((Multipart) contentPart.getContent());
			}
			// text/plain 또는 text/html인 경우
			else {
				message.setContent(contentPart.getContent(), contentPart
						.getContentType());
			}
		}

		return message;
	}

	private Message[] getJavaMailMessageList(Folder folder,
			List<MessageBean> messageList) throws Exception {

		Message[] messageArray = new Message[messageList.size()];
		for (int i = 0; i < messageList.size(); i++) {

			MessageBean messageBean = messageList.get(i);
			messageArray[i] = folder.getMessage(messageBean.getNo().intValue());
		}

		return messageArray;
	}

	private MessageAttachBean getMessageAttach(BodyPart part,
			String multipartPath, boolean inputStream) throws Exception {

		MessageAttachBean attachBean = new MessageAttachBean();
		attachBean.setFileName(JavaMailUtil.decodeText(part.getFileName(),
				JavaMailUtil.getContentCharset(part)));
		attachBean.setFileSize(part.getSize());
		attachBean.setFileType(JavaMailUtil.getContentMimeType(part));
		attachBean.setMultipartPath(multipartPath);

		// Content-ID를 저장한다. 메세지에 embeded 된 이미지에서 주로 사용한다.
		// HTML에 작성자가 직접 태그를 기술하여 넣은 이미지를 말하는것은 아니다.
		// 아웃룩 등에서 이미지를 본문에 넣으면 <img src="cid:컨텐츠아이디"> 와 같이 만들어진다.
		// HangaStudio도 메일 작성시 CID를 이용한 embeded image를 지원할 예정이다.
		// 메일 본문을 보여줄때도 <img src="cid:...">이 있으면 치환해서 보여주어야한다.
		String cid = JavaMailUtil.getHeaderFirst(part, "Content-ID");
		attachBean.setCid(cid);

		if (inputStream) {
			attachBean.setInputStream(part.getInputStream());
		}

		return attachBean;
	}

	private void setFlagFromMessage(MessageBean destBean, Message message)
			throws Exception {

		Flags flags = message.getFlags();

		/*
		 * System Flag (메일 시스템 기본 플래그)
		 */
		Flag[] flagArray = flags.getSystemFlags();
		for (Flag flag : flagArray) {

			if (flag == Flag.SEEN) {
				destBean.setRead(Boolean.valueOf(true));
			}
		}

		/*
		 * User Flag (사용자 정의 플래그)
		 */
		String[] userFlagArray = flags.getUserFlags();
		for (String flag : userFlagArray) {

			if (flag.equals(MailConstants.MSG_USER_FLAG_LOCK)) {
				destBean.setLock(Boolean.valueOf(true));
			}
		}

		flagArray = null;
		userFlagArray = null;
	}

	/**
	 * JavaMail의 Multipart를 MessageBean으로 변환하여 반환한다.<br />
	 * 메세지의 Header정보는 외부에서 따로 저장해야한다. <br />
	 * 주의: 재귀함수가 호출되는 부분이 있다. (MESSAGE/RFC822가 있을경우)
	 * 
	 * @param multipart
	 * @return MessageBean
	 * @throws Exception
	 */
	private MessageBean getMessage(Multipart multipart, String folderName,
			String parentPath) throws Exception {

		MessageBean messageBean = new MessageBean();

		// 본문으로 사용될 Part
		BodyPart textPart = null;

		for (int i = 0; i < multipart.getCount(); i++) {

			// 자기 자신의 path를 저장한다. 첨부파일용
			String path = parentPath;
			if (StringUtil.isEmpty(path)) {
				path = "";
			} else {
				path += "|";
			}
			path += String.valueOf(i);

			boolean attach = false;

			BodyPart part = multipart.getBodyPart(i);

			// Disposition이 INLINE or ATTACHMENT이면 첨부파일이다.
			// 현재는 Text또는 HTML에서만 확인한다.
			String dispo = part.getDisposition();
			if (!StringUtil.isEmpty(dispo)
					&& (dispo.equalsIgnoreCase(Part.INLINE) || dispo
							.equalsIgnoreCase(Part.ATTACHMENT))) {
				attach = true;
			}

			if (part.isMimeType(JavaMailUtil.MULTIPART_ALTERNATIVE)) {
				textPart = JavaMailUtil.getTextBodyPart((Multipart) part
						.getContent());
			}
			// 이 두가지를 모두 보낼땐 multipart/alternative로 보내는게 일반적이므로 or 체크를 해도 된다.
			// 상단에서 Dispose등을 확인하여 첨부파일이 아닌 경우에만 textPart로 지정한다.
			else if (!attach
					&& (part.isMimeType(JavaMailUtil.TEXT_PLAIN) || part
							.isMimeType(JavaMailUtil.TEXT_HTML))) {
				textPart = part;
			}
			// RFC822. "전달" 기능에 주로 사용된다. 서브메세지 정도의 의미. 기본 헤더와 틀은 다 갖추고 있다.
			else if (part.isMimeType(JavaMailUtil.MESSAGE_RFC822)) {

				Object content = part.getContent();
				if (content != null && content instanceof Message) {

					// 재귀함수를 호출한다. 최대한 조심하자.
					MessageBean rfcMessageBean = getMessage((Message) content,
							folderName, true, path);
					if (rfcMessageBean != null) {
						messageBean.addChildMessageList(rfcMessageBean);
					}
				}

				content = null;
			}
			// 나머지는 전부 첨부파일로 간주한다.
			else {

				MessageAttachBean attachBean = getMessageAttach(part, path,
						false);
				messageBean.addAttachList(attachBean);
			}

			part = null;
		}

		// multipart/alternative, text/plain, text/html 셋중 하나인 경우
		if (textPart != null) {

			messageBean.setContentMimeType(JavaMailUtil
					.getContentMimeType(textPart));
			messageBean.setContent((String) textPart.getContent());

			textPart = null;
		}

		return messageBean;
	}

	/**
	 * JavaMail의 Message를 MessageBean으로 변환하여 반환한다.<br>
	 * Header와 Body 모두 저장한다.
	 * 
	 * @param message
	 * @param body
	 *            Body의 추출여부 (목록 : false, 내용보기 : true)
	 * @return MessageBean
	 * @throws Exception
	 */
	private MessageBean getMessage(Message message, String folderName,
			boolean body, String parentPath) throws Exception {

		// 아래 body내에서 multipart/mixed(등)를 만나면 새로 생성하여 반환된 객체를 사용.
		MessageBean messageBean = new MessageBean();

		if (body) {

			Part textPart = null;

			if (message.isMimeType("multipart/*")) {

				Multipart multipart = (Multipart) message.getContent();

				// 같은내용의 본문을 여러형식으로 보낸것 (text/plain + text/html 등)
				if (message.isMimeType(JavaMailUtil.MULTIPART_ALTERNATIVE)) {
					textPart = JavaMailUtil.getTextBodyPart(multipart);
				}
				// multipart/mixed(등).
				else {
					messageBean = getMessage(multipart, folderName, parentPath);
				}
			} else {
				textPart = (Part) message;
			}

			// multipart가 아니거나, multipart/alternative인 경우.
			if (textPart != null) {

				messageBean.setContentMimeType(JavaMailUtil
						.getContentMimeType(textPart));
				messageBean.setContent((String) textPart.getContent());

				textPart = null;
			}

		}

		// 헤더에서 제목을 가져온다. getSubject는 디코딩되므로 사용하지 않는다.
		// JavaMailUtil.decodeText는 비표준 헤더를 최대한 디코딩 하기 위해 사용한다.
		String subject = JavaMailUtil.decodeText(JavaMailUtil.getHeaderFirst(
				message, "Subject"), JavaMailUtil.getContentCharset(message));
		// Index 0 : 이름, Index 1 : 이메일 주소
		String[] from = JavaMailUtil.getFrom(message);

		messageBean.setNo(Integer.valueOf(message.getMessageNumber()));
		messageBean.setFromName(from[0]);
		messageBean.setFromAddress(from[1]);
		messageBean.setSubject(subject);
		messageBean.setDate(message.getSentDate());
		messageBean.setSize(message.getSize());
		messageBean.setFolderName(folderName);

		setFlagFromMessage(messageBean, message);

		subject = null;
		from = null;

		return messageBean;
	}

	private Message[] getMessageArray(Folder openedFolder,
			MessageBean messageBean, boolean paging) throws Exception {

		// 폴더를 오픈하는 비용을 줄이기위해 무조건 밖에서 오픈된 상태로만 작동한다.
		if (!openedFolder.isOpen()) {
			throw new Exception("folder not opened");
		}

		Message[] messageArray = null;

		// 페이징 처리를 하는경우 잘라서 가져온다. startNo와 endNo는 Action쪽에서 처리
		if (paging) {
			messageArray = openedFolder.getMessages(messageBean.getStartNo()
					.intValue(), messageBean.getEndNo().intValue());
		} else {
			messageArray = openedFolder.getMessages();
		}

		return messageArray;

	}

	/*
	 * 외부에서 호출하는 메소드. 주석은 인터페이스에 있음
	 */

	private void setRecipients(Message destMessage, MessageBean messageBean)
			throws Exception {

		// 받는사람 주소
		destMessage.addRecipients(Message.RecipientType.TO, InternetAddress
				.parse(messageBean.getTo(), false));
		// 참조 주소
		if (!StringUtil.isEmpty(messageBean.getCc())) {
			destMessage.addRecipients(Message.RecipientType.CC, InternetAddress
					.parse(messageBean.getCc(), false));
		}
		// 숨은 참조 주소
		if (!StringUtil.isEmpty(messageBean.getBcc())) {
			destMessage.addRecipients(Message.RecipientType.BCC,
					InternetAddress.parse(messageBean.getBcc(), false));
		}
	}

	// encoding을 messageBean에 담을수도 있겠으나 받은 객체의 값을 바꿀경우 외부에서 문제가 생길 수 있다.
	private void setAttaches(MimeMultipart destMultipart,
			MessageBean messageBean, String encoding) throws Exception {

		if (!CollectionUtil.isEmpty(messageBean.getAttachList())) {

			// 첨부파일을 multipart에 넣는다.
			for (MessageAttachBean bean : messageBean.getAttachList()) {

				String fileName = MimeUtility.encodeText(bean.getFileName(),
						encoding, "B");

				MimeBodyPart bodyPart = new MimeBodyPart();
				bodyPart.attachFile(bean.getFilePath());
				// 임시파일명과 실제 파일명이 다른경우를 대비하여 파일명을 따로 넣는다.
				if (!StringUtil.isEmpty(fileName)) {
					bodyPart.setFileName(fileName);
				}

				destMultipart.addBodyPart(bodyPart);

				bodyPart = null;
				fileName = null;
			}
		}
	}

	private void sendMessage(Session session, AccSendBean accSendBean,
			Message message) throws Exception {

		SMTPTransport transport = null;

		try {

			transport = (SMTPTransport) session
					.getTransport(MailConstants.PROTOCOL_SEND);

			// 인증을 거쳐 SMTP를 사용하는 경우
			if (BooleanUtil.booleanValue(accSendBean.getAuth())) {
				transport.connect(accSendBean.getId(), accSendBean
						.getPassword());
			} else {
				transport.connect();
			}
			// 메일 발송
			transport.sendMessage(message, message.getAllRecipients());

			if (logger.isDebugEnabled()) {
				logger.debug("transport send message");
				logger.debug("session properties : "
						+ session.getProperties().toString());
			}
		} finally {

			if (transport != null) {
				transport.close();
				transport = null;
			}
		}
	}

	private void deleteMessageList(Folder openedFolder,
			List<MessageBean> messageList) throws Exception {

		// 폴더를 오픈하는 비용을 줄이기위해 무조건 밖에서 오픈된 상태로만 작동한다.
		if (!openedFolder.isOpen()) {
			throw new Exception("Folder not opened");
		}

		for (MessageBean messageBean : messageList) {

			Message message = openedFolder.getMessage(messageBean.getNo()
					.intValue());
			message.setFlag(Flags.Flag.DELETED, true);
		}

		openedFolder.expunge();
	}

	private SearchTerm mergeTerm(SearchTerm oldTerm, SearchTerm term, boolean or) {

		if (oldTerm != null) {

			if (or) {
				oldTerm = new OrTerm(oldTerm, term);
			} else {
				oldTerm = new AndTerm(oldTerm, term);
			}

			return oldTerm;
		} else {
			return term;
		}
	}

	private SearchTerm getSearchTerm(MessageBean messageBean, boolean or) {

		SearchTerm searchTerm = null;

		// 제목
		if (!StringUtil.isEmpty(messageBean.getSubject())) {
			searchTerm = new SubjectTerm(messageBean.getSubject());
		}
		// 보낸이
		if (!StringUtil.isEmpty(messageBean.getFromName())) {
			searchTerm = mergeTerm(searchTerm, new FromStringTerm(messageBean
					.getFromName()), or);
		}
		// 날짜
		if (messageBean.getDate() != null) {
			searchTerm = mergeTerm(searchTerm, new SentDateTerm(
					ComparisonTerm.EQ, messageBean.getDate()), or);
		}

		return searchTerm;
	}

	public MessageBean getMessage(AccRecvBean accRecvBean,
			MessageBean messageBean) throws Exception {

		// JavaMail Store를 래핑한 Bean을 생성한다.
		MailStoreBean storeBean = new MailStoreBean(accRecvBean);
		Folder folder = null;

		try {

			folder = storeBean.getOpenedFolder(messageBean.getFolderName(),
					Folder.READ_ONLY);

			Message message = folder.getMessage(messageBean.getNo().intValue());
			if (message == null) {
				return null;
			} else {
				return getMessage(message, messageBean.getFolderName(), true,
						null);
			}
		} finally {
			folder = JavaMailUtil.closeFolder(folder);
			storeBean = storeBean.close();
		}

	}

	public List<MessageBean> getMessageList(AccRecvBean accRecvBean,
			MessageBean messageBean) throws Exception {

		// JavaMail Store를 래핑한 Bean을 생성한다.
		MailStoreBean storeBean = new MailStoreBean(accRecvBean);
		Folder folder = null;

		try {

			folder = storeBean.getOpenedFolder(messageBean.getFolderName(),
					Folder.READ_ONLY);

			Message[] messageArray = null;

			SearchTerm searchTerm = getSearchTerm(messageBean, false);
			if (searchTerm != null) {
				messageArray = folder.search(searchTerm);
			} else {
				messageArray = getMessageArray(folder, messageBean, true);
			}

			List<MessageBean> messageList = new ArrayList<MessageBean>();
			for (int i = messageArray.length - 1; i >= 0; i--) {
				messageList.add(getMessage(messageArray[i], messageBean
						.getFolderName(), false, null));
			}

			messageArray = null;
			searchTerm = null;

			return messageList;
		} finally {
			folder = JavaMailUtil.closeFolder(folder);
			storeBean = storeBean.close();
		}

	}

	public Integer getMessageCount(AccRecvBean accRecvBean,
			MessageBean messageBean) throws Exception {

		// JavaMail Store를 래핑한 Bean을 생성한다.
		MailStoreBean storeBean = new MailStoreBean(accRecvBean);
		Folder folder = null;

		try {

			folder = storeBean.getOpenedFolder(messageBean.getFolderName(),
					Folder.READ_ONLY);

			int messageCount = 0;

			SearchTerm searchTerm = getSearchTerm(messageBean, false);
			if (searchTerm != null) {

				// TODO folder.search후에 folder.getMessageCount()의 개수가 변하는지확인하여
				// 변한다면 아래 Array를 이용한 개수 계산은 뺀다.
				Message[] messageArray = folder.search(searchTerm);
				if (messageArray != null) {

					messageCount = messageArray.length;
					messageArray = null;
				}

				searchTerm = null;
			} else {
				messageCount = folder.getMessageCount();
			}

			return messageCount;
		} finally {
			folder = JavaMailUtil.closeFolder(folder);
			storeBean = storeBean.close();
		}
	}

	public void insertMessage(AccSendBean accSendBean, AccRecvBean accRecvBean,
			MessageBean messageBean) throws Exception {

		/*
		 * 준비작업
		 */

		Properties props = new Properties();
		props.put("mail." + MailConstants.PROTOCOL_SEND + ".host", accSendBean
				.getHost());
		props.put("mail." + MailConstants.PROTOCOL_SEND + ".port", accSendBean
				.getPort());
		props.put("mail." + MailConstants.PROTOCOL_SEND + ".auth", String
				.valueOf(BooleanUtil.booleanValue(accSendBean.getAuth())));

		if (BooleanUtil.booleanValue(accSendBean.getSsl())) {
			SSLInitializer.init();
			JavaMailUtil.setPropertiesForSsl(props,
					MailConstants.PROTOCOL_SEND, accSendBean.getSslPort());
		}

		Session session = Session.getInstance(props, null);

		/*
		 * 메세지 생성
		 */

		// JavaMail Store를 래핑한 Bean을 생성한다.
		MailStoreBean storeBean = new MailStoreBean(accRecvBean);
		Folder forwardFolder = null;
		Folder saveFolder = null;

		try {

			if (BooleanUtil.booleanValue(messageBean.getForward())
					&& !StringUtil.isEmpty(messageBean.getFolderName())) {
				forwardFolder = storeBean.getOpenedFolder(messageBean
						.getFolderName(), Folder.READ_ONLY);
			}

			Message message = getJavaMailMessage(session, accRecvBean,
					messageBean, forwardFolder);

			// 메일 발송
			sendMessage(session, accSendBean, message);

			// 보낸 편지함에 메일 저장
			if (BooleanUtil.booleanValue(messageBean.getSave())
					&& !StringUtil.isEmpty(messageBean.getSaveFolderName())) {

				saveFolder = storeBean.getOpenedFolder(messageBean
						.getSaveFolderName(), Folder.READ_WRITE);
				saveFolder.appendMessages(new Message[] { message });
			}

			message = null;
		} finally {

			forwardFolder = JavaMailUtil.closeFolder(forwardFolder);
			saveFolder = JavaMailUtil.closeFolder(saveFolder);
			storeBean = storeBean.close();

			props = null;
			session = null;
		}
	}

	public void updateMessageFlag(AccRecvBean accRecvBean,
			MessageBean messageBean) throws Exception {

		// JavaMail Store를 래핑한 Bean을 생성한다.
		MailStoreBean storeBean = new MailStoreBean(accRecvBean);
		Folder folder = null;

		try {

			folder = storeBean.getOpenedFolder(messageBean.getFolderName(),
					Folder.READ_WRITE);

			Message message = folder.getMessage(messageBean.getNo().intValue());

			Flags trueFlags = new Flags();
			Flags falseFlags = new Flags();

			// 읽음
			if (BooleanUtil.booleanValue(messageBean.getRead())) {
				trueFlags.add(Flag.SEEN);
			} else {
				falseFlags.add(Flag.SEEN);
			}

			// 잠금
			if (BooleanUtil.booleanValue(messageBean.getLock())) {
				trueFlags.add(MailConstants.MSG_USER_FLAG_LOCK);
			} else {
				falseFlags.add(MailConstants.MSG_USER_FLAG_LOCK);
			}

			message.setFlags(trueFlags, true);
			message.setFlags(falseFlags, false);

			trueFlags = null;
			falseFlags = null;
			message = null;
		} finally {
			folder = JavaMailUtil.closeFolder(folder);
			storeBean = storeBean.close();
		}
	}

	public void updateMessageReadFlag(AccRecvBean accRecvBean,
			MessageBean messageBean) throws Exception {

		// JavaMail Store를 래핑한 Bean을 생성한다.
		MailStoreBean storeBean = new MailStoreBean(accRecvBean);
		Folder folder = null;

		try {

			folder = storeBean.getOpenedFolder(messageBean.getFolderName(),
					Folder.READ_WRITE);

			Message message = folder.getMessage(messageBean.getNo().intValue());
			message.setFlag(Flag.SEEN, BooleanUtil.booleanValue(messageBean
					.getRead()));

			message = null;
		} finally {
			folder = JavaMailUtil.closeFolder(folder);
			storeBean = storeBean.close();
		}
	}

	public void deleteMessageList(AccRecvBean accRecvBean,
			MessageBean messageBean) throws Exception {

		// JavaMail Store를 래핑한 Bean을 생성한다.
		MailStoreBean storeBean = new MailStoreBean(accRecvBean);
		Folder folder = null;

		try {

			folder = storeBean.getOpenedFolder(messageBean.getFolderName(),
					Folder.READ_WRITE);

			deleteMessageList(folder, messageBean.getChildMessageList());
		} finally {
			folder = JavaMailUtil.closeFolder(folder);
			storeBean = storeBean.close();
		}
	}

	public void moveMessageList(AccRecvBean accRecvBean, MessageBean messageBean)
			throws Exception {

		// JavaMail Store를 래핑한 Bean을 생성한다.
		MailStoreBean storeBean = new MailStoreBean(accRecvBean);
		Folder folder = null;
		Folder saveFolder = null;

		try {

			Folder defaultFolder = storeBean.getDefaultFolder();
			folder = defaultFolder.getFolder(messageBean.getFolderName());
			saveFolder = defaultFolder.getFolder(messageBean
					.getSaveFolderName());

			// JavaMail 폴더를 READ_WRITE로 연다.
			folder.open(Folder.READ_WRITE);
			folder.copyMessages(getJavaMailMessageList(folder, messageBean
					.getChildMessageList()), saveFolder);

			deleteMessageList(folder, messageBean.getChildMessageList());

			defaultFolder = null;
		} finally {
			folder = JavaMailUtil.closeFolder(folder);
			storeBean = storeBean.close();
		}

	}

	public MessageAttachBean getMessageAttach(AccRecvBean accRecvBean,
			MessageBean messageBean, MessageAttachBean attachBean)
			throws Exception {

		// JavaMail Store를 래핑한 Bean을 생성한다.
		MailStoreBean storeBean = new MailStoreBean(accRecvBean);
		Folder folder = null;

		try {

			folder = storeBean.getOpenedFolder(messageBean.getFolderName(),
					Folder.READ_ONLY);

			Message message = folder.getMessage(messageBean.getNo().intValue());

			BodyPart part = null;
			if (message.isMimeType("multipart/*")) {

				Multipart multipart = (Multipart) message.getContent();

				// MessageAttachBean에 저장된 해당 BodyPart의 경로를 "|"로 잘라서 배열에 넣는다.
				String[] pathArray = attachBean.getMultipartPath().split("[|]");
				part = JavaMailUtil.getBodyPart(multipart, pathArray);

				pathArray = null;
				multipart = null;
			}

			if (part == null) {
				return null;
			} else {
				return getMessageAttach(part, attachBean.getMultipartPath(),
						false);
			}
		} finally {
			folder = JavaMailUtil.closeFolder(folder);
			storeBean = storeBean.close();
		}
	}
}
