/* 
 * This file is part of hanga studio.
 *
 * Copyright (C) 2007 hanga studio Project.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
 */
package kr.sohalee.hangastudio.presentation.service.mail;

import java.util.List;

import kr.sohalee.hangastudio.constants.ConfigConstants;
import kr.sohalee.hangastudio.constants.MailConstants;
import kr.sohalee.hangastudio.core.cache.CacheManager;
import kr.sohalee.hangastudio.core.exception.BizConnectException;
import kr.sohalee.hangastudio.core.util.BooleanUtil;
import kr.sohalee.hangastudio.core.util.IntegerUtil;
import kr.sohalee.hangastudio.model.config.ConfigCacheMap;
import kr.sohalee.hangastudio.model.mail.MailCacheMap;
import kr.sohalee.hangastudio.model.mail.MailStoreBean;
import kr.sohalee.hangastudio.model.mail.acc.AccRecvBean;
import kr.sohalee.hangastudio.model.mail.acc.AccSendBean;
import kr.sohalee.hangastudio.model.mail.acc.AccService;
import kr.sohalee.hangastudio.model.user.UserBean;
import kr.sohalee.hangastudio.presentation.service.ServiceSupport;

/**
 * - Mail에서 Action Class들이 공통으로 사용하는 부분들을 위해 제작되었다.<br /> - ExecutePreparable을
 * 구현하였으므로 <b>executePrepare</b> 메소드를 오버라이드하려면 반드시 <b>super.executePrepare()</b>를
 * 실행한다.<br />
 * 
 * @author <a href="mailto:sohalee@dreamwiz.com">Lee, Kyung-rok</a>
 * 
 */
@SuppressWarnings("serial")
public class MailSupport extends ServiceSupport {

	protected AccService accService = null;

	public void setMailAccService(AccService accService) {
		this.accService = accService;
	}

	/**
	 * 기본 계정이 설정되어 있는지 확인한다. 기본 계정 설정 없이 메일을 사용할 수 없다.
	 * 
	 * @return
	 * @throws Exception
	 */
	protected boolean validDefaultAccRecv() throws Exception {

		AccRecvBean defaultRecvBean = getSessDefaultAccRecvBean();
		if (defaultRecvBean == null) {
			addActionError("error.mail.recvDefaultNotFound");
			return false;
		}

		MailStoreBean store = null;
		try {
			store = new MailStoreBean(defaultRecvBean);
		} catch (BizConnectException e) {

			addActionError("error.mail.recvDefaultConnectTest");
			addActionError(e.getMessage());

			return false;
		} finally {

			if (store != null) {
				store.close();
				store = null;
			}

			defaultRecvBean = null;
		}

		return true;
	}

	/**
	 * 메일 관련 캐쉬 맵을 반환한다.
	 * 
	 * @return MailCacheMap 메일 관련 캐쉬 맵
	 * @throws Exception
	 */
	public MailCacheMap getMailCacheMap() throws Exception {
		return (MailCacheMap) CacheManager.getInstance().get(
				MailCacheMap.class.getName());
	}

	/**
	 * 세션에 저장된 받는 서버 목록
	 * 
	 * @return
	 * @throws Exception
	 */
	@SuppressWarnings("unchecked")
	public List<AccRecvBean> getSessAccRecvList() throws Exception {

		if (getSessionObject(MailConstants.SESS_ACC_RECV_LIST) == null) {

			UserBean userBean = getSessUserBean();

			AccRecvBean recvBean = new AccRecvBean();
			recvBean.setUserId(userBean.getUserId());
			setSessionObject(MailConstants.SESS_ACC_RECV_LIST, accService
					.getAccRecvList(recvBean));

			userBean = null;
		}

		return (List) getSessionObject(MailConstants.SESS_ACC_RECV_LIST);
	}

	/**
	 * 받는 메일 서버중 기본으로 설정된 Bean을 반환한다.
	 * 
	 * @return AccRecvBean 받는 메일 서버 Bean
	 */
	public AccRecvBean getSessDefaultAccRecvBean() throws Exception {

		/*
		 * 기본 계정은 세션에 넣어 가져오게 되면 계정 등록 관리시 Default에 변동이 있으면 제어해주어야한다. 제어하는것 자체는
		 * 어렵지 않으나 기본 계정은 메일 발송시 이메일 주소를 만들어내는등 중요하게 작동하니 확실하게 매번 찾아서 가져오자 계정
		 * 변동시 List는 항상 갱신해준다.
		 */
		for (AccRecvBean recvBean : getSessAccRecvList()) {
			if (BooleanUtil.booleanValue(recvBean.getDefault())) {
				return recvBean;
			}
		}

		return null;
	}

	/**
	 * 받는 메일 서버 정보를 반환한다.
	 * 
	 * @return AccRecvBean 받는 메일 서버 Bean
	 */
	public AccRecvBean getSessAccRecvBean() throws Exception {

		if (getSessionObject(MailConstants.SESS_ACC_RECV) == null) {
			setSessionObject(MailConstants.SESS_ACC_RECV,
					getSessDefaultAccRecvBean());
		}

		return (AccRecvBean) getSessionObject(MailConstants.SESS_ACC_RECV);
	}

	protected boolean changeSessAccRecvBean(AccRecvBean accRecvBean)
			throws Exception {

		for (AccRecvBean recvBean : getSessAccRecvList()) {
			if (IntegerUtil.compareIntValue(accRecvBean.getAccRecvSeq(),
					recvBean.getAccRecvSeq())) {
				setSessionObject(MailConstants.SESS_ACC_RECV, recvBean);
				return true;
			}
		}

		return false;
	}

	/**
	 * 보내는 메일 서버정보를 반환한다.
	 * 
	 * @return AccSendBean 보내는 메일 서버 Bean
	 */
	public AccSendBean getSessAccSendBean() throws Exception {

		/*
		 * 테스트코드입니다. 나중에 정상 구현후 삭제해야합니다.
		 */
		if (getSessionObject(MailConstants.SESS_ACC_SEND) == null) {

			UserBean userBean = getSessUserBean();

			AccSendBean sendBean = new AccSendBean();
			sendBean.setId(userBean.getSmtpId());
			sendBean.setPassword(userBean.getSmtpPassword());
			sendBean.setHost(ConfigCacheMap
					.getConfigValue(ConfigConstants.CODE_MAIL_SMTP_HOST));
			sendBean.setPort(ConfigCacheMap
					.getConfigValue(ConfigConstants.CODE_MAIL_SMTP_PORT));
			sendBean.setAuth(Boolean.valueOf(ConfigCacheMap
					.getConfigValue(ConfigConstants.CODE_MAIL_SMTP_AUTH)));
			sendBean.setSsl(Boolean.valueOf(ConfigCacheMap
					.getConfigValue(ConfigConstants.CODE_MAIL_SMTP_SSL)));
			sendBean.setSslPort(ConfigCacheMap
					.getConfigValue(ConfigConstants.CODE_MAIL_SMTP_SSL_PORT));

			setSessionObject(MailConstants.SESS_ACC_SEND, sendBean);
		}

		return (AccSendBean) getSessionObject(MailConstants.SESS_ACC_SEND);
	}
}
